==> cf-report-image-toolbox-1.0.7.tgz: package/src/api/git-providers/github/github.api.ts <==
import { Octokit } from '@octokit/rest'
import _ from 'lodash'

import { CodefreshAPI } from '../../codefresh'
import { GitEnricherInputs } from '../../../image-enricher-git-info/types'

export class GithubApi {
    private githubCreds
    private inputs
    private codefresh: CodefreshAPI

    constructor(inputs: GitEnricherInputs) {
        this.inputs = inputs
        this.codefresh = new CodefreshAPI(inputs.cfHost, inputs.cfApiKey)
    }
    async getBranch(repo, branch) {
        const apiClient = await this.prepareApiClient()
        const [ repoOwner, repoName ] = this.getRepoOwnerAndName(repo)

        const response = await apiClient.repos.getBranch({
            owner: repoOwner,
            repo: repoName,
            branch,
        })
        return response.data
    }

    async getCommit(repo, branch, revision) {
        const apiClient = await this.prepareApiClient()
        const [ repoOwner, repoName ] = this.getRepoOwnerAndName(repo)

        const response = await apiClient.repos.getCommit({
            owner: repoOwner,
            repo: repoName,
            ref: revision,
        })
        return response.data
    }

    async searchForPullRequests(repo, branch) {
        const apiClient = await this.prepareApiClient()

        console.log(`Looking for PRs from "${repo}" repo and "${branch}" branch`)

        const response = await apiClient.search.issuesAndPullRequests({ q: `head:${branch}+type:pr+repo:${repo}` })
        return response.data
    }

    async getCommitsInfo(repo, pullRequestId) {
        const apiClient = await this.prepareApiClient()
        const [ repoOwner, repoName ] = this.getRepoOwnerAndName(repo)

        const commitsByUserLimit = this.inputs.commitsByUserLimit

        const committersMap = {}
        const commitsByUser = {}
        let firstCommitDate
        let lastCommitDate

        let page = 1
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        while (true) {
            const commits = await apiClient.pulls.listCommits({
                owner: repoOwner,
                repo: repoName,
                // eslint-disable-next-line @typescript-eslint/naming-convention
                pull_number: pullRequestId,
                page,
            })

            if (commits.data.length === 0) {
                break
            }

            if (page === 1) {
                firstCommitDate = commits.data[0].commit.author?.date
            }

            lastCommitDate = _.last(commits.data)?.commit.author?.date

            for (const commit of commits.data) {
                const userName = commit.author?.login || _.get(commit, 'commit.committer.name')
                if (!userName) {
                    continue
                }

                committersMap[userName] = {
                    userName,
                    avatar: commit.author?.avatar_url,
                }

                if (!commitsByUser[userName]) {
                    commitsByUser[userName] = []
                }

                commitsByUser[userName].push({
                    url: commit.html_url,
                    userName,
                    sha: commit.sha,
                    message: commit.commit.message,
                    commitDate: commit.commit.author?.date
                })
            }

            page++
        }

        for (const userName of Object.keys(commitsByUser)) {
            commitsByUser[userName] = _.takeRight(commitsByUser[userName], commitsByUserLimit)
        }


        return {
            committers: _.values(committersMap),
            commits: _.flatten(_.values(commitsByUser)),
            firstCommitDate,
            lastCommitDate
        }
    }

    private async prepareApiClient() {
        const { githubApiHost, githubApiPathPrefix, githubToken } = await this.resolveCreds()

        const octokit = new Octokit({
            auth: githubToken,
            baseUrl: this.buildRequestUrl(githubApiHost, githubApiPathPrefix)
        })

        return octokit
    }

    private async resolveCreds() {
        if (this.githubCreds) {
            return this.githubCreds
        }

        if (this.inputs.githubToken) {
            this.githubCreds = {
                githubToken: this.inputs.githubToken,
                githubApiHost: this.inputs.githubApiHost || 'api.github.com',
                githubApiPathPrefix: this.inputs.githubApiPathPrefix || '/'
            }
        } else if (this.inputs.githubContextName) {
            const context = await this.codefresh.getContext(this.inputs.githubContextName)
            const githubToken = context.spec.data.auth.password
            const githubApiPathPrefix = _.get(context, 'spec.data.auth.apiPathPrefix', '/')
            const githubApiHost = _.get(context, 'spec.data.auth.apiHost', 'api.github.com')
            this.githubCreds = { githubApiHost, githubApiPathPrefix, githubToken }
        } else {
            throw new Error('GitHub credentials not provided')
        }
        return this.githubCreds
    }

    private buildRequestUrl(apiHost, apiPathPrefix) {
        try {
            let pathPrefix = apiPathPrefix.replace(/\/$/, '') // remove the last slash
            if (pathPrefix && pathPrefix[0] !== '/') {
                pathPrefix = `/${pathPrefix}` // ensure left slash
            }

            // Sanitizing URL for supporting all formats existing in DB
            // Normalize parts of url in format: (host)(/pathPrefix)
            let host = apiHost.replace(/\/$/, '') // remove the last slash
            if (!/^http(s?)/.test(host)) {
                host = `https://${host}`
            }

            return `${host}${pathPrefix}`
        } catch (error) {
            console.error(error)
            throw new Error(`github api host is not valid`)
        }
    }

    private getRepoOwnerAndName(repo) {
        const [ owner, ...name ] = repo.split('/')
        return [ owner, name.join('/') ]
    }
}

==> cf-report-image-toolbox-1.0.10.tgz: package/src/api/git-providers/github/github.api.ts <==
import { Octokit } from '@octokit/rest'
import _ from 'lodash'

import { GitEnricherInputs } from '../../../image-enricher-git-info/types'

export class GithubApi {
    private inputs

    constructor(inputs: GitEnricherInputs) {
        this.inputs = inputs
    }
    async getBranch(repo, branch) {
        const apiClient = await this.prepareApiClient()
        const [ repoOwner, repoName ] = this.getRepoOwnerAndName(repo)

        const response = await apiClient.repos.getBranch({
            owner: repoOwner,
            repo: repoName,
            branch,
        })
        return response.data
    }

    async getCommit(repo, branch, revision) {
        const apiClient = await this.prepareApiClient()
        const [ repoOwner, repoName ] = this.getRepoOwnerAndName(repo)

        const response = await apiClient.repos.getCommit({
            owner: repoOwner,
            repo: repoName,
            ref: revision,
        })
        return response.data
    }

    async searchForPullRequests(repo, branch) {
        const apiClient = await this.prepareApiClient()

        console.log(`Looking for PRs from "${repo}" repo and "${branch}" branch`)

        const response = await apiClient.search.issuesAndPullRequests({ q: `head:${branch}+type:pr+repo:${repo}` })
        return response.data
    }

    async getCommitsInfo(repo, pullRequestId) {
        const apiClient = await this.prepareApiClient()
        const [ repoOwner, repoName ] = this.getRepoOwnerAndName(repo)

        const commitsByUserLimit = this.inputs.commitsByUserLimit

        const committersMap = {}
        const commitsByUser = {}
        let firstCommitDate
        let lastCommitDate

        let page = 1
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        while (true) {
            const commits = await apiClient.pulls.listCommits({
                owner: repoOwner,
                repo: repoName,
                // eslint-disable-next-line @typescript-eslint/naming-convention
                pull_number: pullRequestId,
                page,
            })

            if (commits.data.length === 0) {
                break
            }

            if (page === 1) {
                firstCommitDate = commits.data[0].commit.author?.date
            }

            lastCommitDate = _.last(commits.data)?.commit.author?.date

            for (const commit of commits.data) {
                const userName = commit.author?.login || _.get(commit, 'commit.committer.name')
                if (!userName) {
                    continue
                }

                committersMap[userName] = {
                    userName,
                    avatar: commit.author?.avatar_url,
                }

                if (!commitsByUser[userName]) {
                    commitsByUser[userName] = []
                }

                commitsByUser[userName].push({
                    url: commit.html_url,
                    userName,
                    sha: commit.sha,
                    message: commit.commit.message,
                    commitDate: commit.commit.author?.date
                })
            }

            page++
        }

        for (const userName of Object.keys(commitsByUser)) {
            commitsByUser[userName] = _.takeRight(commitsByUser[userName], commitsByUserLimit)
        }


        return {
            committers: _.values(committersMap),
            commits: _.flatten(_.values(commitsByUser)),
            firstCommitDate,
            lastCommitDate
        }
    }

    private async prepareApiClient() {
        const githubToken = this.inputs.githubToken
        const githubApiHost = this.inputs.githubApiHost || 'api.github.com'
        const githubApiPathPrefix = this.inputs.githubApiPathPrefix || '/'
        const octokit = new Octokit({
            auth: githubToken,
            baseUrl: this.buildRequestUrl(githubApiHost, githubApiPathPrefix)
        })

        return octokit
    }

    private buildRequestUrl(apiHost, apiPathPrefix) {
        try {
            let pathPrefix = apiPathPrefix.replace(/\/$/, '') // remove the last slash
            if (pathPrefix && pathPrefix[0] !== '/') {
                pathPrefix = `/${pathPrefix}` // ensure left slash
            }

            // Sanitizing URL for supporting all formats existing in DB
            // Normalize parts of url in format: (host)(/pathPrefix)
            let host = apiHost.replace(/\/$/, '') // remove the last slash
            if (!/^http(s?)/.test(host)) {
                host = `https://${host}`
            }

            return `${host}${pathPrefix}`
        } catch (error) {
            console.error(error)
            throw new Error(`github api host is not valid`)
        }
    }

    private getRepoOwnerAndName(repo) {
        const [ owner, ...name ] = repo.split('/')
        return [ owner, name.join('/') ]
    }
}
